package net.puppygames.jglib.util;

import riven.classpath.math.Vec2;

public strictfp class EasyMath {
	public static final float FULL_CIRCLE = (float) (Math.PI * 2.0);

	public static boolean isPowerOfTwo(int n) {
		return ((n != 0) && ((n & (~n + 1)) == n));
	}

	public static int ceilToPowerOfTwo(int n) {
		int h = Integer.highestOneBit(n);
		int l = Integer.lowestOneBit(n);
		return 1 << (h + (h == l ? 0 : 1));
	}

	public static boolean isWithinManhattanRange(int x, int y, int maxDistance) {
		//return Math.max(Math.abs(x), Math.abs(y)) <= maxDistance;
		return (x * x) <= (maxDistance * maxDistance) && (y * y) <= (maxDistance * maxDistance);
	}

	public static boolean isWithinManhattanRange(float x, float y, float maxDistance) {
		//return Math.max(Math.abs(x), Math.abs(y)) <= maxDistance;
		return (x * x) <= (maxDistance * maxDistance) && (y * y) <= (maxDistance * maxDistance);
	}

	public static boolean isWithinRange(int x, int y, int maxDistance) {
		return x * x + y * y <= (maxDistance * maxDistance);
	}

	public static boolean isWithinRange(float x, float y, float maxDistance) {
		return x * x + y * y <= (maxDistance * maxDistance);
	}

	public static boolean isWithinRange(double x, double y, double maxDistance) {
		return x * x + y * y <= (maxDistance * maxDistance);
	}

	public static float hypot(int dx, int dy) {
		return (float) Math.sqrt(dx * dx + dy * dy);
	}

	public static float lerp(float a, float b, float ratio) {
		return a + ratio * (b - a);
	}

	public static float invlerp(float cur, float min, float max) {
		return (cur - min) / (max - min);
	}

	public static float invlerp(long cur, long min, long max) {
		return (cur - min) / (float) (max - min);
	}

	//

	public static float interpolate(float cur, float min, float max, float start, float end) {
		return lerp(invlerp(cur, min, max), start, end);
	}

	public static float interpolate(long cur, long min, long max, float start, float end) {
		return lerp(invlerp(cur, min, max), start, end);
	}

	public static void interpolate(long cur, long min, long max, Vec2 start, Vec2 end, Vec2 out) {
		float ratio = invlerp(cur, min, max);
		out.load(lerp(ratio, start.x, end.x), lerp(ratio, start.y, end.y));
	}

	//

	public static float cappedInterpolate(float cur, float min, float max, float start, float end) {
		return (cur < min) ? start : (cur > max) ? end : interpolate(cur, min, max, start, end);
	}

	public static float cappedInterpolate(long cur, long min, long max, float start, float end) {
		return (cur < min) ? start : (cur > max) ? end : interpolate(cur, min, max, start, end);
	}

	public static void cappedInterpolate(long cur, long min, long max, Vec2 start, Vec2 end, Vec2 out) {
		if (cur < min) {
			out.load(start);
		} else if (cur > max) {
			out.load(end);
		} else {
			interpolate(cur, min, max, start, end, out);
		}
	}

	//

	public static float clamp(float value) {
		return (value < 0.0f) ? 0.0f : (value > 1.0f ? 1.0f : value);
	}

	public static float clamp(float value, float min, float max) {
		return (value < min) ? min : (value > max ? max : value);
	}

	public static int clamp(int value, int min, int max) {
		return (value < min) ? min : (value > max ? max : value);
	}

	public static int clampIndex(int index, int[] arr) {
		if (arr.length == 0) {
			throw new IllegalStateException();
		}
		return clamp(index, 0, arr.length - 1);
	}

	public static int fetchClamped(int index, int[] arr) {
		if (arr.length == 0) {
			throw new IllegalStateException();
		}
		index = clamp(index, 0, arr.length - 1);
		return arr[index];
	}
}
