package net.puppygames.tools;

import org.lwjgl.LWJGLException;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;

import riven.classpath.OpenGLCommons;

import static org.lwjgl.opengl.GL11.*;

public class DemoFramework {
	public static int WINDOW_WIDTH = 1024;
	public static int WINDOW_HEIGHT = 768;

	public static void main(String[] args) {

	}

	public final void mainLoop() {
		try {
			Display.setDisplayMode(new DisplayMode(WINDOW_WIDTH, WINDOW_HEIGHT));
			Display.setFullscreen(false);
			Display.setResizable(true);
			Display.setTitle(this.getClass().getSimpleName());

			createDisplay();
		} catch (LWJGLException exc) {
			System.out.println(exc);
			return;
		}

		this.setup();

		this.resized();

		int frameCounter = 0;
		long lastSecond = System.nanoTime();
		while (!Display.isCloseRequested()) {
			this.input();

			if (Display.wasResized()) {
				this.resized();
			}

			this.preRender();
			this.render();

			// finish frame
			Display.update();
			frameCounter += 1;
			if (lastSecond + 1_000_000_000L < System.nanoTime()) {
				Display.setTitle(this.getClass().getSimpleName() + " - " + frameCounter + "fps");
				lastSecond += 1_000_000_000L;
				frameCounter = 0;
			}
			if (this.doSync()) {
				Display.sync(60);
			}
		}

		Display.destroy();

		System.exit(0);
	}

	protected void createDisplay() throws LWJGLException {
		Display.create();
	}

	protected boolean doSync() {
		return true;
	}

	protected void input() {

	}

	protected void setup() {

	}

	protected void resized() {

	}

	protected void preRender() {
		glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
		OpenGLCommons.initOrthoProjectionModelViewMatrix(WINDOW_WIDTH, WINDOW_HEIGHT);

		glClearColor(0, 0, 0, 1);
		glClear(GL_COLOR_BUFFER_BIT);
	}

	protected void render() {

	}
}
