package net.puppygames.tools;

import java.nio.FloatBuffer;

import net.puppygames.jglib.util.EasyMath;

import org.lwjgl.BufferUtils;
import org.lwjgl.LWJGLException;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.PixelFormat;

import riven.classpath.math.Matrix4;
import riven.classpath.math.Vec3;

import static org.lwjgl.opengl.GL11.*;

import static org.lwjgl.util.glu.GLU.*;

public class DemoFramework3D extends DemoFramework {
	public static void main(String[] args) {
		System.out.println("DemoFramework3D");
		new DemoFramework3D().mainLoop();
	}

	public static final float ONE_UNIT_PER_SECOND = 1.0f / 60.0f;

	public float cameraAngleH;
	public float cameraAngleV;
	public final Vec3 cameraPosition = new Vec3();
	public final float playerRotationFactor = 0.5f;
	public final float playerWalkVelocity = 4.0f; // m/s
	public final Vec3 playerPosOld = new Vec3(), playerPosNew = new Vec3();

	@Override
	protected void createDisplay() throws LWJGLException {
		Display.create(new PixelFormat(8, 24, 8, 8));
	}

	@Override
	protected void setup() {
		playerPosOld.load(1.0f, 0.0f, 1.0f);
		playerPosNew.load(playerPosOld);

		cameraAngleH = 45.0f; // 0 == look at Z axis
		cameraAngleV = +15.0f; // look down a bit
	}

	private boolean firstMouseEventDXY = true;
	public Vec3 movement = new Vec3(0, 0, 0);

	@Override
	protected void input() {
		while (Mouse.next()) {
			// sigh...
			int mouseDX = Mouse.getEventDX();
			int mouseDY = Mouse.getEventDY();
			if (mouseDX != 0 || mouseDY != 0) {
				if (firstMouseEventDXY) {
					firstMouseEventDXY = false;
					continue;
				}
			}

			onMouseEvent();
		}

		while (Keyboard.next()) {
			onKeyboardEvent();
		}

		movement.load(0, 0, 0);
		{
			if (Keyboard.isKeyDown(Keyboard.KEY_W)) {
				movement.add(0, 0, +1);
			}
			if (Keyboard.isKeyDown(Keyboard.KEY_S)) {
				movement.add(0, 0, -1);
			}
			if (Keyboard.isKeyDown(Keyboard.KEY_A)) {
				movement.add(-1, 0, 0);
			}
			if (Keyboard.isKeyDown(Keyboard.KEY_D)) {
				movement.add(+1, 0, 0);
			}
			if (!movement.isZero()) {
				movement.normalize();
				movement.mul(ONE_UNIT_PER_SECOND);
				movement.mul(playerWalkVelocity);

				Matrix4 mat = new Matrix4();
				mat.rotY(cameraAngleH);
				mat.transform(movement);
			}
		}
	}

	protected void onMouseEvent() {
		if (Mouse.isButtonDown(0)) {
			cameraAngleH += Mouse.getEventDX() * +playerRotationFactor;
			cameraAngleH += 180.0f;
			cameraAngleH %= 360.0f;
			cameraAngleH -= 180.0f;

			cameraAngleV += Mouse.getEventDY() * -playerRotationFactor;
			cameraAngleV = EasyMath.clamp(cameraAngleV, -45, +85);
		}

		if (Mouse.getEventButton() == 0) {
			Mouse.setGrabbed(Mouse.getEventButtonState());
		}
	}

	protected void onKeyboardEvent() {

	}

	Matrix4 modelViewMatrix = new Matrix4();

	@Override
	protected void preRender() {
		glClearColor(0, 0, 0, 0);
		glColorMask(true, true, true, true);

		glClearStencil(0x00);
		glStencilMask(0xff);

		glClearDepth(1.0f);
		glDepthMask(true);

		glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT);
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

		glMatrixMode(GL_PROJECTION);
		{
			float fieldOfView = 60.0f;
			float aspectRatio = (float) WINDOW_WIDTH / WINDOW_HEIGHT;
			float zNear = 0.001f;
			float zFar = 100.0f;

			glLoadIdentity();
			gluPerspective(fieldOfView, aspectRatio, zNear, zFar);
		}

		glMatrixMode(GL_MODELVIEW);
		{
			glLoadIdentity();

			// we're building the view part of the modelview matrix in reverse:
			{
				// translate
				// rotate Y
				// rotate X
				// scale
			}
			glScalef(+1, +1, -1); // ensure our Z axis points to the far-plane 
			glRotatef(-cameraAngleV, 1, 0, 0);
			glRotatef(-cameraAngleH, 0, 1, 0);
			glTranslatef(-cameraPosition.x, -cameraPosition.y, -cameraPosition.z);

			//
			FloatBuffer fbuf = BufferUtils.createFloatBuffer(16);
			glGetFloat(GL_MODELVIEW_MATRIX, fbuf);
			modelViewMatrix.setTranspose(fbuf);
		}

		glEnable(GL_DEPTH_TEST);
		glDepthFunc(GL_LEQUAL);

		glEnable(GL_CULL_FACE);
		glFrontFace(GL_CCW);
		glCullFace(GL_BACK);
	}

	@Override
	protected void render() {
		cameraPosition.add(movement);

		glBegin(GL_QUADS);

		glColor3f(1, 0, 0);
		renderCube(-5, -5, -5, 10, 10, 10, TOP | BOTTOM);

		glColor3f(0, 1, 1);
		renderCube(-5, -5, -5, 10, 10, 10, NORTH | SOUTH);

		glColor3f(1, 1, 0);
		renderCube(-5, -5, -5, 10, 10, 10, EAST | WEST);

		glEnd();
	}

	private static final int BOTTOM = 1 << 0;
	private static final int NORTH = 1 << 1;
	private static final int SOUTH = 1 << 2;
	private static final int EAST = 1 << 3;
	private static final int WEST = 1 << 4;
	private static final int TOP = 1 << 5;

	public void renderCube(float x, float y, float z, float w, float h, float d, int mask) {
		Vec3 nw1 = new Vec3(w, 0, 0).add(x, y, z);
		Vec3 ne1 = new Vec3(w, 0, d).add(x, y, z);
		Vec3 sw1 = new Vec3(0, 0, 0).add(x, y, z);
		Vec3 se1 = new Vec3(0, 0, d).add(x, y, z);
		Vec3 nw2 = new Vec3(w, h, 0).add(x, y, z);
		Vec3 ne2 = new Vec3(w, h, d).add(x, y, z);
		Vec3 sw2 = new Vec3(0, h, 0).add(x, y, z);
		Vec3 se2 = new Vec3(0, h, d).add(x, y, z);

		if ((mask & BOTTOM) != 0) {
			glVertex(nw1);
			glVertex(ne1);
			glVertex(se1);
			glVertex(sw1);
		}

		if ((mask & NORTH) != 0) {
			glVertex(nw2);
			glVertex(ne2);
			glVertex(ne1);
			glVertex(nw1);
		}

		if ((mask & SOUTH) != 0) {
			glVertex(sw1);
			glVertex(se1);
			glVertex(se2);
			glVertex(sw2);
		}

		if ((mask & EAST) != 0) {
			glVertex(ne2);
			glVertex(se2);
			glVertex(se1);
			glVertex(ne1);
		}

		if ((mask & WEST) != 0) {
			glVertex(nw1);
			glVertex(sw1);
			glVertex(sw2);
			glVertex(nw2);
		}

		if ((mask & TOP) != 0) {
			glVertex(sw2);
			glVertex(se2);
			glVertex(ne2);
			glVertex(nw2);
		}
	}

	static void glVertex(Vec3 vec) {
		glVertex3f(vec.x, vec.y, vec.z);
	}

	static void glColor(Vec3 color) {
		glColor3f(color.x, color.y, color.z);
	}
}
